/*
* light weight WS2812 lib V2.1 - Arduino support
*
* Controls WS2811/WS2812/WS2812B RGB-LEDs
* Author: Matthias Riegler
*
* Mar 07 2014: Added Arduino and C++ Library
*
* License: GNU GPL v2 (see License.txt)
*/

#ifndef WS2812_H_
#define WS2812_H_

#include <avr/interrupt.h>
#include <avr/io.h>
#ifndef F_CPU
#define  F_CPU 16000000UL
#endif
#include <util/delay.h>
#include <stdint.h>

#ifdef ARDUINO
#if (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#include <pins_arduino.h>
#endif
#endif


#define RED()      WS2812_LED(0, 0x80, 0x00, 0x00);LED.sync();
#define GREEN()    WS2812_LED(0, 0x00, 0x80, 0x00);LED.sync();
#define BLUE()     WS2812_LED(0, 0x00, 0x00, 0x80);LED.sync();
#define MAGENTA()  WS2812_LED(0, 0x80, 0x00, 0x80);LED.sync();
#define CYAN()     WS2812_LED(0, 0x00, 0x80, 0x80);LED.sync();
#define YELLOW()   WS2812_LED(0, 0x80, 0x80, 0x00);LED.sync();
#define LEDS_OFF() WS2812_LED(0, 0x00, 0x00, 0x00);LED.sync();

struct cRGB { uint8_t g; uint8_t r; uint8_t b; };

class WS2812 {
public: 
	WS2812(uint16_t num_led);
	~WS2812();
	
	#ifndef ARDUINO
	void setOutput(const volatile uint8_t* port, volatile uint8_t* reg, uint8_t pin);
	#else
	void setOutput(uint8_t pin);
	#endif
	
	cRGB get_crgb_at(uint16_t index);
	uint8_t set_crgb_at(uint16_t index, cRGB px_value);
	uint8_t setColorRGB(uint16_t index, uint8_t red, uint8_t green, uint8_t blue);
	void sync();
	void render();
	
private:
	uint16_t count_led;
	uint8_t *pixels;
	
	void ws2812_sendarray_mask(uint8_t *array,uint16_t length, uint8_t pinmask,uint8_t *port, uint8_t *portreg);

	const volatile uint8_t *ws2812_port;
	volatile uint8_t *ws2812_port_reg;
	uint8_t pinMask; 
};



#endif /* WS2812_H_ */